"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _utils = _interopRequireDefault(require("../utils"));

var _local_storage_adapter = _interopRequireDefault(require("./local_storage_adapter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class PersistentQueueGoogleAnalyticsAdapter {
  constructor(storage = _local_storage_adapter.default) {
    _defineProperty(this, "namespace", "alephbet");

    _defineProperty(this, "queue_name", "_ga_queue");

    this._storage = storage;
    this._queue = JSON.parse(this._storage.get(this.queue_name) || "[]");

    this._flush();
  }

  _remove_uuid(uuid) {
    return () => {
      _utils.default.remove(this._queue, el => el.uuid === uuid);

      this._storage.set(this.queue_name, JSON.stringify(this._queue));
    };
  }

  _flush() {
    if (typeof global.ga !== "function") {
      throw new Error("ga not defined. " + "Please make sure your Universal analytics is set up correctly");
    }

    for (const item of this._queue) {
      const callback = this._remove_uuid(item.uuid);

      global.ga("send", "event", item.category, item.action, item.label, {
        hitCallback: callback,
        nonInteraction: 1
      });
    }
  }

  _track(category, action, label) {
    _utils.default.log("Persistent Queue Google Universal Analytics track", {
      category,
      action,
      label
    });

    if (this._queue.length > 100) this._queue.shift();

    this._queue.push({
      uuid: _utils.default.uuid(),
      category,
      action,
      label
    });

    this._storage.set(this.queue_name, JSON.stringify(this._queue));

    this._flush();
  }

  experiment_start(experiment, variant) {
    this._track(this.namespace, `${experiment.name} | ${variant}`, "Visitors");
  }

  goal_complete(experiment, variant, goal_name, _props) {
    this._track(this.namespace, `${experiment.name} | ${variant}`, goal_name);
  }

}

var _default = PersistentQueueGoogleAnalyticsAdapter;
exports.default = _default;