"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _local_storage_adapter = _interopRequireDefault(require("./local_storage_adapter"));

var _utils = _interopRequireDefault(require("../utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class PersistentQueueKeenAdapter {
  constructor(keen_client, storage = _local_storage_adapter.default) {
    _defineProperty(this, "queue_name", "_keen_queue");

    this.client = keen_client;
    this._storage = storage;
    this._queue = JSON.parse(this._storage.get(this.queue_name) || "[]");

    this._flush();
  }

  _remove_quuid(quuid) {
    return (err, _res) => {
      if (err) return;

      _utils.default.remove(this._queue, el => el.properties._quuid === quuid);

      this._storage.set(this.queue_name, JSON.stringify(this._queue));
    };
  }

  _flush() {
    for (const item of Array.from(this._queue)) {
      const callback = this._remove_quuid(item.properties._quuid);

      this.client.addEvent(item.experiment_name, _utils.default.omit(item.properties, "_quuid"), callback);
    }
  }

  _user_uuid(experiment, goal) {
    if (!experiment.user_id) return _utils.default.uuid(); // if goal is not unique, we track it every time. return a new random uuid

    if (!goal.unique) return _utils.default.uuid(); // for a given user id, namespace and experiment, the uuid will always be the same
    // this avoids counting goals twice for the same user across different devices

    return _utils.default.sha1(`${this.namespace}.${experiment.name}.${experiment.user_id}`);
  }

  _track(experiment, variant, goal) {
    _utils.default.log("Persistent Queue Keen track", {
      experiment: experiment.name,
      variant,
      goal: goal.name
    });

    if (this._queue.length > 100) this._queue.shift();

    this._queue.push({
      experiment_name: experiment.name,
      properties: {
        _quuid: _utils.default.uuid(),
        // queue uuid (used only internally)
        uuid: this._user_uuid(experiment, goal),
        variant,
        event: goal.name
      }
    });

    this._storage.set(this.queue_name, JSON.stringify(this._queue));

    this._flush();
  }

  experiment_start(experiment, variant) {
    this._track(experiment, variant, {
      name: "participate",
      unique: true
    });
  }

  goal_complete(experiment, variant, goal_name, props) {
    this._track(experiment, variant, _utils.default.defaults({
      name: goal_name
    }, props));
  }

}

var _default = PersistentQueueKeenAdapter;
exports.default = _default;