function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import LocalStorageAdapter from "./local_storage_adapter";
import utils from "../utils";

var PersistentQueueKeenAdapter = /*#__PURE__*/function () {
  function PersistentQueueKeenAdapter(keen_client) {
    var storage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : LocalStorageAdapter;

    _classCallCheck(this, PersistentQueueKeenAdapter);

    _defineProperty(this, "queue_name", "_keen_queue");

    this.client = keen_client;
    this._storage = storage;
    this._queue = JSON.parse(this._storage.get(this.queue_name) || "[]");

    this._flush();
  }

  _createClass(PersistentQueueKeenAdapter, [{
    key: "_remove_quuid",
    value: function _remove_quuid(quuid) {
      var _this = this;

      return function (err, _res) {
        if (err) return;
        utils.remove(_this._queue, function (el) {
          return el.properties._quuid === quuid;
        });

        _this._storage.set(_this.queue_name, JSON.stringify(_this._queue));
      };
    }
  }, {
    key: "_flush",
    value: function _flush() {
      for (var _i = 0, _Array$from = Array.from(this._queue); _i < _Array$from.length; _i++) {
        var item = _Array$from[_i];

        var callback = this._remove_quuid(item.properties._quuid);

        this.client.addEvent(item.experiment_name, utils.omit(item.properties, "_quuid"), callback);
      }
    }
  }, {
    key: "_user_uuid",
    value: function _user_uuid(experiment, goal) {
      if (!experiment.user_id) return utils.uuid(); // if goal is not unique, we track it every time. return a new random uuid

      if (!goal.unique) return utils.uuid(); // for a given user id, namespace and experiment, the uuid will always be the same
      // this avoids counting goals twice for the same user across different devices

      return utils.sha1("".concat(this.namespace, ".").concat(experiment.name, ".").concat(experiment.user_id));
    }
  }, {
    key: "_track",
    value: function _track(experiment, variant, goal) {
      utils.log("Persistent Queue Keen track", {
        experiment: experiment.name,
        variant: variant,
        goal: goal.name
      });
      if (this._queue.length > 100) this._queue.shift();

      this._queue.push({
        experiment_name: experiment.name,
        properties: {
          _quuid: utils.uuid(),
          // queue uuid (used only internally)
          uuid: this._user_uuid(experiment, goal),
          variant: variant,
          event: goal.name
        }
      });

      this._storage.set(this.queue_name, JSON.stringify(this._queue));

      this._flush();
    }
  }, {
    key: "experiment_start",
    value: function experiment_start(experiment, variant) {
      this._track(experiment, variant, {
        name: "participate",
        unique: true
      });
    }
  }, {
    key: "goal_complete",
    value: function goal_complete(experiment, variant, goal_name, props) {
      this._track(experiment, variant, utils.defaults({
        name: goal_name
      }, props));
    }
  }]);

  return PersistentQueueKeenAdapter;
}();

export default PersistentQueueKeenAdapter;